#!/usr/bin/env python

import sys
import os
import datetime
import time
from stat import *
from urlparse import urlparse
from urllib import url2pathname

class podcastItem:
    def __init__(self, id = None, podcastTitle = None, episodeTitle = None, episodeLocalUrl = None, isNew = None):
        self.id = id
        self.podcastTitle = podcastTitle
        self.episodeTitle = episodeTitle
        self.episodeLocalUrl  = episodeLocalUrl
        self.filePath = None
        if str(isNew) == "0":
            self.isNew = False
        else:
            self.isNew = True
        

    def getFilePath(self):
        if self.filePath != None:
            return self.filePath

        if self.episodeLocalUrl == None:
            return None

        self.filePath = url2pathname(urlparse(self.episodeLocalUrl)[2])
        return self.filePath

    def getFileSize(self):
        if self.episodeLocalUrl == None:
            return None

        try:
            self.fileSize = os.stat(self.getFilePath())[ST_SIZE]
            return self.fileSize

        except:
            return None


    def getFileModTime(self):
        if self.episodeLocalUrl == None:
            return None

        try:
            return os.stat(self.getFilePath())[ST_MTIME]

        except:
            return None


class amarokLocalPodcasts:
    def __init__(self):
        self.cmd = "dcop amarok collection query \"SELECT podcastchannels.title, podcastepisodes.id, podcastepisodes.title, podcastepisodes.localurl, podcastepisodes.isNew FROM podcastchannels, podcastepisodes WHERE podcastchannels.url = podcastepisodes.parent AND podcastepisodes.localurl like 'file:%';\"" 
        self.queryResults =  os.popen(self.cmd).readlines()
        self.podcastsWithLocalFiles = []

        rows = self.queryResults

        while len(rows) > 0:
            podcast_title = rows.pop(0).rstrip("\n\r")
            ep_id         = rows.pop(0).rstrip("\n\r")
            ep_title      = rows.pop(0).rstrip("\n\r")
            ep_file       = rows.pop(0).rstrip("\n\r")
            ep_new        = rows.pop(0).rstrip("\n\r")
            
            p = podcastItem(ep_id, podcast_title, ep_title, ep_file, ep_new)
            if p.getFileSize() != None:
                self.podcastsWithLocalFiles.append(p)

    def getTotalSize(self, ageUnits = None, ageQuantity = None, listenedOnly = False):
        size = 0
        for p in self.getList(ageUnits, ageQuantity, listenedOnly):
            size += p.getFileSize()

        return size

    def getCount(self, ageUnits = None, ageQuantity = None, listenedOnly = False):
        return len(self.getList(ageUnits, ageQuantity, listenedOnly))

    def getDetails(self, ageUnits = None, ageQuantity = None, listenedOnly = False):
        "returns a string containing a listing of podcast files suitable for confirmation messages"
        details = ""
        for p in self.getList(ageUnits, ageQuantity, listenedOnly):
            details += p.podcastTitle
            details += " / "
            details += p.episodeTitle
            details += ' (%.1f Mib)\n' % (p.getFileSize() / (1024.0*1024)) 
            
        return details
        

    def cleanup(self, ageUnits = None, ageQuantity = None, listenedOnly = False):
        for p in self.getList(ageUnits, ageQuantity, listenedOnly):
            os.unlink(p.getFilePath())        

    def getList(self, ageUnits = None, ageQuantity = None, listenedOnly = False):
        if ageUnits == None or ageQuantity == None:
            return self.podcastsWithLocalFiles

        self.filterSeconds = 0L

        # if a filter is specified, filter the list.
        if ageUnits == "minutes":
            self.filterSeconds = ageQuantity * 60
        elif ageUnits == "hours":
            self.filterSeconds = ageQuantity * 3600
        elif ageUnits == "days":
            self.filterSeconds = ageQuantity * 3600 * 24
        elif ageUnits == "weeks":
            self.filterSeconds = ageQuantity * 3600 * 24 * 7
        elif ageUnits == "months":
            # it's a crappy assumption that a month is 30 days, but heh
            self.filterSeconds = ageQuantity * 3600 * 24 * 30
        else:
            raise NameError, "unknown age unit, " + ageUnits 

        runTime = time.mktime(datetime.datetime.now().timetuple())

        self.filteredList = []
        for p in self.podcastsWithLocalFiles:
            if p.getFileModTime() < (runTime - self.filterSeconds):
                if not listenedOnly or not p.isNew:
                    self.filteredList.append(p)

        return self.filteredList
        
if __name__ == '__main__':
    
    pods = amarokLocalPodcasts()
    d = 0
    lo = True
    print pods.getDetails("days", d, lo)
    print "number of files: " + str(pods.getCount("days", d, lo))
    print "size of files:   " + str(pods.getTotalSize("days", d, lo))
    
        
