#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
############################################################################
# Python-Qt podcast cleanup 0.1 script for amaroK
# by Matthew Gates
#
# Basic script structure imspired by / blatantly copied from nightIMPgale
# by ugh_bough
#
# Depends on: Python 2.2, PyQt
############################################################################
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
############################################################################

import threading
import os
import sys
import signal
import datetime
import time
from checker import amarokLocalPodcasts, podcastItem
import ConfigParser
from configuredialog import ConfigureDialog

try:
    from qt import *
    
except:
    os.system("kdialog --sorry 'PyQt (Qt bindings for Python) is required for this script.'")
    raise

############################################################################
############################################################################

class Config:
    __INST = None 
    __CONFIGFILE = "settings.ini"

    def instance():
        if Config.__INST == None:
            Config.__INST = Config()
            # print "Created Config singleton"

        return Config.__INST

    instance = staticmethod(instance)
	
    def __init__(self):
        self.config = ConfigParser.ConfigParser()
        if os.path.isfile(Config.__CONFIGFILE):
            # print "reading settings from " + Config.__CONFIGFILE
            self.read()
        else:
            # print "using default settings"
            self.setDefaults()        
        
    def read(self):
        try:
            # read config
            self.config.read(Config.__CONFIGFILE)
            
        except:
            # set standard
            os.system("dcop amarok playlist popupMessage 'podcast cleanup: failed to read settings file'")
            # os.system("kdialog --msgbox 'podcast cleanup: failed to read settings file'")

    def setDefaults(self):
        if not self.config.has_section("on_start"):
            self.config.add_section("on_start")
            
        self.config.set("on_start", "enabled", "False")
        self.config.set("on_start", "filter", "listened")
        self.config.set("on_start", "condition_units", "hours")
        self.config.set("on_start", "condition_num", "4")
        self.config.set("on_start", "confirm", "False")
        self.config.set("on_start", "notify", "True")
            
        if not self.config.has_section("scan"):
            self.config.add_section("scan")
            
        self.config.set("scan", "enabled", "False")
        self.config.set("scan", "scan_time_units", "hours")
        self.config.set("scan", "scan_time_num", "1")
        self.config.set("scan", "filter", "listened")
        self.config.set("scan", "condition_units", "hours")
        self.config.set("scan", "condition_num", "4")
        self.config.set("scan", "confirm", "False")
        self.config.set("scan", "notify", "True")
			
    def write(self):
        try:
            f = file(Config.__CONFIGFILE, 'w')
            self.config.write(f)
            f.close()
            
        except:
            os.system("dcop amarok playlist popupMessage 'podcast cleanup: error saving settings file'")


############################################################################

class ConfigDialog(ConfigureDialog):
    def __init__(self):
        ConfigureDialog.__init__(self)
        self.setWFlags(Qt.WDestructiveClose)
        self.applySettingsToGui()
        self.getSummary()

    def applySettingsToGui(self):
        self.deleteOnStartCheckbox.setChecked(Config.instance().config.getboolean("on_start", "enabled"))
        if Config.instance().config.get("on_start", "filter") == "all":
            self.startupRadioAll.setChecked(True)
        elif Config.instance().config.get("on_start", "filter") == "listened":
            self.startupRadioListened.setChecked(True)
        elif Config.instance().config.get("on_start", "filter") == "byage":
            self.startupRadioOlder.setChecked(True)
        else:
            print "Unknown on_start.filter value: " + Config.instance().config.get("on_start", "filter")

        self.startupNumSpin.setValue(Config.instance().config.getint("on_start", "condition_num"))
        self.startupAgeUnits.setCurrentText(Config.instance().config.get("on_start", "condition_units"))
        self.confirmDeletionStartupCheck.setChecked(Config.instance().config.getboolean("on_start", "confirm"))
        self.notifyDeletionStartupCheck.setChecked(Config.instance().config.getboolean("on_start", "notify"))

        # second page of settings - periondic scanning
        self.scheduleCheckbox.setChecked(Config.instance().config.getboolean("scan", "enabled"))
        self.scheduleTimeSpin.setValue(Config.instance().config.getint("scan", "scan_time_num"))
        self.scheduleAgeCombo.setCurrentText(Config.instance().config.get("scan", "scan_time_units"))
        
        if Config.instance().config.get("scan", "filter") == "all":
            self.scheduleRadioAll.setChecked(True)
        elif Config.instance().config.get("scan", "filter") == "listened":
            self.scheduleRadioListened.setChecked(True)
        elif Config.instance().config.get("scan", "filter") == "byage":
            self.scheduleRadioOlder.setChecked(True)
        else:  
            print "Unknown scan.filter value: " + Config.instance().config.get("scan", "filter")

        self.scheduleNumSpin.setValue(Config.instance().config.getint("scan", "condition_num"))
        self.scheduleAgeUnitsCombo.setCurrentText(Config.instance().config.get("scan", "condition_units"))

        self.confirmDeletionScanCheck.setChecked(Config.instance().config.getboolean("scan", "confirm"))
        self.notifyDeletionScanCheck.setChecked(Config.instance().config.getboolean("scan", "notify"))

    # called when a configuration section is selected from the
    # listbox on the left of the form
    def changeForm(self):
        self.widgetStack1.raiseWidget(self.listBox.currentItem())

    # called when the delete all now button is clicked
    def deleteAllPodcastFilesNow(self):
        if not os.system("kdialog --yesno 'Are you sure you want to clean up all downloaded podcast files?'"):
            PodChecker.instance().cleanup()
            
    # called to populate the summary information
    def getSummary(self):
        self.summaryNumFiles.setText( str(PodChecker.instance().getCount()) )
        self.summarySize.setText( str('%.1f' % (PodChecker.instance().getTotalSize() / (1024.0*1024))) )
        

    def setDefaultSettings(self):
        Config.instance().setDefaults()
        self.applySettingsToGui()

    # public slot
    def saveSettings(self):
        # reads the values from the form, sets the values in the Config.config
        # object and tells it to save to file.
        # first page - startup
        Config.instance().config.set("on_start", "enabled", str(self.deleteOnStartCheckbox.isChecked()))
        if self.startupRadioAll.isChecked():
            Config.instance().config.set("on_start", "filter", "all")
        elif self.startupRadioListened.isChecked():
            Config.instance().config.set("on_start", "filter", "listened")
        elif self.startupRadioOlder.isChecked():
            Config.instance().config.set("on_start", "filter", "byage")
        else:
            print "Don't know how to set on_start.filter"

        Config.instance().config.set("on_start", "condition_units", str(self.startupAgeUnits.currentText()))
        Config.instance().config.set("on_start", "condition_num", str(self.startupNumSpin.value()))
        Config.instance().config.set("on_start", "confirm", str(self.confirmDeletionStartupCheck.isChecked()))
        Config.instance().config.set("on_start", "notify", str(self.notifyDeletionStartupCheck.isChecked()))

        # second page - periodic scanning
        Config.instance().config.set("scan", "enabled", str(self.scheduleCheckbox.isChecked()))
        Config.instance().config.set("scan", "scan_time_units", str(self.scheduleAgeCombo.currentText()))
        Config.instance().config.set("scan", "scan_time_num", str(self.scheduleTimeSpin.value()))
        if self.scheduleRadioAll.isChecked():
            Config.instance().config.set("scan", "filter", "all")
        elif self.scheduleRadioListened.isChecked():
            Config.instance().config.set("scan", "filter", "listened")
        elif self.scheduleRadioOlder.isChecked():
            Config.instance().config.set("scan", "filter", "byage")
        else:
            print "Don't know how to set scan.filter config option"

        Config.instance().config.set("scan", "condition_units", str(self.scheduleAgeUnitsCombo.currentText()))
        Config.instance().config.set("scan", "condition_num", str(self.scheduleNumSpin.value()))
        Config.instance().config.set("scan", "confirm", str(self.confirmDeletionScanCheck.isChecked()))
        Config.instance().config.set("scan", "notify", str(self.notifyDeletionScanCheck.isChecked()))
        Config.instance().write()

        self.accept()

############################################################################

class Notification(QCustomEvent):
    __super_init = QCustomEvent.__init__
    def __init__(self, string):
        self.__super_init(QCustomEvent.User + 1)
        self.string = string

############################################################################

class PodChecker(amarokLocalPodcasts):
    __INST = None 

    def instance():
        if PodChecker.__INST == None:
            PodChecker.__INST = PodChecker()
            # print "Created PodChecker singleton"

        return PodChecker.__INST

    instance = staticmethod(instance)

    def __init__(self):
        amarokLocalPodcasts.__init__(self)
        self.lastClean = time.mktime(datetime.datetime.now().timetuple())

	    
    def doCleanup(self, ageUnit, ageQuantity, onlyListened, confirm, notify):
        fileCount = PodChecker.instance().getCount(ageUnit, ageQuantity, onlyListened)
        fileSize  = '%.1f' % (PodChecker.instance().getTotalSize(ageUnit, ageQuantity, onlyListened) / (1024.0*1024))

        if fileCount <= 0:
            self.lastClean = time.mktime(datetime.datetime.now().timetuple())
            return

        if confirm:
            cmd = "kdialog --yesno 'podcast cleanup: are you sure you want to clean up " + str(fileCount) + " files (" + fileSize + "MiB)?'"
            if os.system(cmd):
                self.lastClean = time.mktime(datetime.datetime.now().timetuple())
                return

        PodChecker.instance().cleanup(ageUnit, ageQuantity, onlyListened)
        self.lastClean = time.mktime(datetime.datetime.now().timetuple())

        if notify:
            cmd = "dcop amarok playlist popupMessage 'podcast cleanup: deleted " + str(fileCount) + " files (" + fileSize + "MiB)'"
            # cmd = "kdialog --msgbox 'podcast cleanup: deleted " + str(fileCount) + " files (" + fileSize + "MiB)'"
            os.system(cmd)

############################################################################

class PodcastCleanup(QApplication):
    def __init__(self, args):
        if Config.instance().config.getboolean("on_start", "enabled"):
            self.startupClean()
        else:
            self.lastClean = time.mktime(datetime.datetime.now().timetuple())
        
        QApplication.__init__(self, args)
        threading.Thread(target = self.readStdin).start()

    def readConfig(self):
        Config.instance().read()

    def readStdin(self):
        while True:
            line = sys.stdin.readline()
            if line:
                qApp.postEvent(self, Notification(line))
            else:
                break

        # os.system("kdialog --msgbox 'podcast cleanup: amarok seems to have closed communication with me'")
        os._exit(0)

    def customEvent(self, notification):
        string = QString(notification.string)
        if string.contains("configure"):
            self.configure()

    def configure(self):
        self.dialog = ConfigDialog()
        self.dialog.show()
        self.connect(self.dialog, SIGNAL("destroyed()"), self.readConfig)

    def startupClean(self):
        # print "doing startup cleaning..."
        confirm = Config.instance().config.getboolean("on_start", "confirm")
        notify = Config.instance().config.getboolean("on_start", "notify")

        if Config.instance().config.get("on_start", "filter") == "all":
            PodChecker.instance().doCleanup(None, None, False, confirm, notify)
        elif Config.instance().config.get("on_start", "filter") == "listened":
            PodChecker.instance().doCleanup(None, None, True, confirm, notify)
        elif Config.instance().config.get("on_start", "filter") == "byage":
            PodChecker.instance().doCleanup(
                Config.instance().config.get("on_start", "condition_units"),
                Config.instance().config.getint("on_start", "condition_num"),
                True,
                confirm,
                notify)
                

        

############################################################################

def main():
    app = PodcastCleanup(sys.argv)
    app.exec_loop()
    
def onStop(signum, stackframe):
    os.system("dcop amarok playlist popupMessage 'podcast cleanup: terminating'")
    # os.system("kdialog --msgbox 'podcast cleanup: terminating'")
    os._exit(0)

if __name__ == "__main__":
    threading.Thread(target=main).start()

    signal.signal(1, onStop)    # sighup
    signal.signal(2, onStop)    # sigint
    signal.signal(15, onStop)   # sigterm

    while True:
        secondsSinceLastCheck = time.mktime(datetime.datetime.now().timetuple()) - PodChecker.instance().lastClean
        age = Config.instance().config.getint("scan", "scan_time_num")
             
        if Config.instance().config.get("scan", "scan_time_units") == "minutes":
             age *= 60
        elif Config.instance().config.get("scan", "scan_time_units") == "hours":
             age *= 3600
        elif Config.instance().config.get("scan", "scan_time_units") == "days":
             age *= 3600 * 24
        elif Config.instance().config.get("scan", "scan_time_units") == "weeks":
             age *= 3600 * 24 * 7
        elif Config.instance().config.get("scan", "scan_time_units") == "months":
             age *= 3600 * 24 * 30
        else:
            raise NameError, "unknown age units for scan frequency"
    
        if secondsSinceLastCheck > age:
            # print "doing scan cleaning..."
            confirm = Config.instance().config.getboolean("scan", "confirm")
            notify = Config.instance().config.getboolean("scan", "notify")

            if Config.instance().config.get("scan", "filter") == "all":
                PodChecker.instance().doCleanup(None, None, False, confirm, notify)
            elif Config.instance().config.get("scan", "filter") == "listened":
                PodChecker.instance().doCleanup(None, None, True, confirm, notify)
            elif Config.instance().config.get("scan", "filter") == "byage":
                PodChecker.instance().doCleanup(
                    Config.instance().config.get("scan", "condition_units"),
                    Config.instance().config.getint("scan", "condition_num"),
                    True,
                    confirm,
                    notify)
        # else:
        #    print "waiting (" + str(secondsSinceLastCheck) + " seconds since last scan - waiting till " + str(age) + ")"

        time.sleep(1)

